@@ -28,17 +28,19 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.TreeSet;
 
 import org.apache.commons.io.FilenameUtils;
+import org.apache.royale.compiler.clients.MXMLJSC.JSTargetType;
 import org.apache.royale.compiler.clients.problems.ProblemPrinter;
 import org.apache.royale.compiler.clients.problems.ProblemQuery;
 import org.apache.royale.compiler.clients.problems.ProblemQueryProvider;
 import org.apache.royale.compiler.clients.problems.WorkspaceProblemFormatter;
 import org.apache.royale.compiler.codegen.js.IJSWriter;
-import org.apache.royale.compiler.codegen.js.goog.IJSGoogPublisher;
+import org.apache.royale.compiler.codegen.js.royale.IJSRoyalePublisher;
 import org.apache.royale.compiler.config.Configuration;
 import org.apache.royale.compiler.config.ConfigurationBuffer;
 import org.apache.royale.compiler.config.Configurator;
@@ -61,6 +63,8 @@ import org.apache.royale.compiler.internal.targets.RoyaleJSTarget;
 import org.apache.royale.compiler.internal.targets.JSTarget;
 import org.apache.royale.compiler.internal.units.ResourceModuleCompilationUnit;
 import org.apache.royale.compiler.internal.units.SourceCompilationUnitFactory;
+import org.apache.royale.compiler.internal.watcher.WatchThread;
+import org.apache.royale.compiler.internal.watcher.WatchThread.IWatchWriter;
 import org.apache.royale.compiler.internal.workspaces.Workspace;
 import org.apache.royale.compiler.problems.ConfigurationProblem;
 import org.apache.royale.compiler.problems.ICompilerProblem;
@@ -101,10 +105,10 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
     {
         SUCCESS(0),
         PRINT_HELP(1),
-        FAILED_WITH_PROBLEMS(0),
-        FAILED_WITH_ERRORS(3),
-        FAILED_WITH_EXCEPTIONS(4),
-        FAILED_WITH_CONFIG_PROBLEMS(5);
+        FAILED_WITH_ERRORS(2),
+        FAILED_WITH_EXCEPTIONS(3),
+        FAILED_WITH_CONFIG_PROBLEMS(4),
+        WATCHING(1000);
 
         ExitCode(int code)
         {
@@ -112,6 +116,11 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
         }
 
         final int code;
+        
+        int getCode()
+        {
+        	return code;
+        }
     }
 
     @Override
@@ -135,7 +144,10 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
     public static void main(final String[] args)
     {
         int exitCode = staticMainNoExit(args);
-        System.exit(exitCode);
+        if (exitCode != ExitCode.WATCHING.getCode())
+        {
+            System.exit(exitCode);
+        }
     }
 
     /**
@@ -163,14 +175,14 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
 
     protected ProblemQuery problems;
     protected ISourceFileHandler asFileHandler;
-    protected Configuration config;
+    protected JSConfiguration config;
     protected Configurator projectConfigurator;
     private ConfigurationBuffer configBuffer;
     private ICompilationUnit mainCU;
     protected ITarget target;
     protected ITargetSettings targetSettings;
     protected IJSApplication jsTarget;
-    private IJSGoogPublisher jsPublisher;
+    private IJSRoyalePublisher jsPublisher;
 
     public MXMLJSCNode()
     {
@@ -179,7 +191,6 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
     
     protected MXMLJSCNode(IBackend backend)
     {
-        DefinitionBase.setPerformanceCachingEnabled(true);
         workspace = new Workspace();
         workspace.setASDocDelegate(new RoyaleASDocDelegate());
         project = new RoyaleJSProject(workspace, backend);
@@ -242,8 +253,8 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
                 {
                     if (problems.hasErrors())
                         exitCode = ExitCode.FAILED_WITH_ERRORS;
-                    else
-                        exitCode = ExitCode.FAILED_WITH_PROBLEMS;
+                    // no exit code for warnings because anything except 0 is
+                    // detected as a failure by various build tools
                 }
             }
             else if (problems.hasFilteredProblems())
@@ -269,7 +280,10 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
         }
         finally
         {
-            waitAndClose();
+            if (!config.getWatch() || !ExitCode.SUCCESS.equals(exitCode))
+            {
+                waitAndClose();
+            }
 
             if (outProblems != null && problems.hasFilteredProblems())
             {
@@ -279,7 +293,71 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
                 }
             }
         }
-        return exitCode.code;
+        if (config.getWatch() && ExitCode.SUCCESS.equals(exitCode))
+        {
+            setupWatcher();
+            exitCode = ExitCode.WATCHING;
+        }
+        return exitCode.getCode();
+    }
+
+    protected void setupWatcher()
+    {
+        if (!config.getWatch())
+        {
+            return;
+        }
+        IWatchWriter writer = new IWatchWriter()
+        {
+            private long startTime;
+
+            public void rebuild(Collection<ICompilationUnit> units, Collection<ICompilerProblem> problems) throws InterruptedException, IOException
+            {
+                startTime = System.nanoTime();
+                workspace.startBuilding();
+                try
+                {
+                    target = project.getBackend().createTarget(project,
+                            getTargetSettings(), null);
+                    ((JSTarget) target).build(mainCU, problems);
+                    for (ICompilationUnit unit : units)
+                    {
+                        // call waitForBuildFinish() to ensure that binding data
+                        // doesn't get lost when a new definition is created
+                        unit.waitForBuildFinish(problems, null);
+                    }
+                }
+                finally
+                {
+                    workspace.doneBuilding();
+                }
+            }
+    
+            public void write(Collection<ICompilationUnit> units) throws InterruptedException, IOException
+            {
+                workspace.startBuilding();
+                try
+                {
+                    File outputFolder = jsPublisher.getOutputFolder();
+                    for (ICompilationUnit unit : units)
+                    {
+                        writeCompilationUnit(unit, outputFolder);
+                    }
+
+                    // ensures that the goog.addDependency() calls are updated
+                    jsPublisher.publish(problems);
+
+                    long endTime = System.nanoTime();
+                    System.out.println((endTime - startTime) / 1e9 + " seconds");
+                }
+                finally
+                {
+                    workspace.doneBuilding();
+                }
+            }
+        };
+        WatchThread watcherThread = new WatchThread(JSTargetType.JS_NODE.getText(), writer, config, project, workspace, problems);
+        watcherThread.start();
     }
 
     /**
@@ -320,7 +398,9 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
                 }
 
                 Set<String> closurePropNamesToKeep = new HashSet<String>();
-                jsPublisher = (IJSGoogPublisher) project.getBackend().createPublisher(
+                //use a LinkedHashSet because the order of the exported names matters -JT
+                LinkedHashSet<String> closureSymbolNamesToExport = new LinkedHashSet<String>();
+                jsPublisher = (IJSRoyalePublisher) project.getBackend().createPublisher(
                         project, errors, config);
 
                 File outputFolder = jsPublisher.getOutputFolder();
@@ -336,62 +416,16 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
 	                ((RoyaleJSTarget)target).collectMixinMetaData(project.mixinClassNames, reachableCompilationUnits);
 	                for (final ICompilationUnit cu : reachableCompilationUnits)
 	                {
-	                    ICompilationUnit.UnitType cuType = cu.getCompilationUnitType();
-	
-	                    if (cuType == ICompilationUnit.UnitType.AS_UNIT
-	                            || cuType == ICompilationUnit.UnitType.MXML_UNIT)
-	                    {
-	                        final File outputClassFile = getOutputClassFile(
-	                                cu.getQualifiedNames().get(0), outputFolder);
-	
-                            if (config.isVerbose())
-                            {
-                                System.out.println("Compiling file: " + outputClassFile);
-                            }
-	
-	                        ICompilationUnit unit = cu;
-	
-	                        IJSWriter writer;
-	                        if (cuType == ICompilationUnit.UnitType.AS_UNIT)
-	                        {
-	                            writer = (IJSWriter) project.getBackend().createWriter(project,
-	                                    errors, unit, false);
-	                        }
-	                        else
-	                        {
-	                            writer = (IJSWriter) project.getBackend().createMXMLWriter(
-	                                    project, errors, unit, false);
-	                        }
-	
-	                        BufferedOutputStream out = new BufferedOutputStream(
-	                                new FileOutputStream(outputClassFile));
-	
-                            BufferedOutputStream sourceMapOut = null;
-	                        File outputSourceMapFile = null;
-	                        if (project.config.getSourceMap())
-	                        {
-	                            outputSourceMapFile = getOutputSourceMapFile(
-	                                    cu.getQualifiedNames().get(0), outputFolder);
-                                sourceMapOut = new BufferedOutputStream(
-                                        new FileOutputStream(outputSourceMapFile));
-	                        }
-	                        
-	                        writer.writeTo(out, sourceMapOut, outputSourceMapFile);
-	                        out.flush();
-	                        out.close();
-                            if (sourceMapOut != null)
-                            {
-                                sourceMapOut.flush();
-                                sourceMapOut.close();
-                            }
-	                        writer.close();
-	                    }
+	                    writeCompilationUnit(cu, outputFolder);
                         ClosureUtils.collectPropertyNamesToKeep(cu, project, closurePropNamesToKeep);
+                        ClosureUtils.collectSymbolNamesToExport(cu, project, closureSymbolNamesToExport);
 	                }
                 }
                 
                 if (jsPublisher != null)
                 {
+                    jsPublisher.setClosurePropertyNamesToKeep(closurePropNamesToKeep);
+                    jsPublisher.setClosureSymbolNamesToExport(closureSymbolNamesToExport);
                     compilationSuccess = jsPublisher.publish(problems);
                 }
                 else
@@ -409,6 +443,61 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
         return compilationSuccess;
     }
 
+    protected void writeCompilationUnit(ICompilationUnit cu, File outputFolder) throws InterruptedException, IOException
+    {
+        ICompilationUnit.UnitType cuType = cu.getCompilationUnitType();
+        if (cuType != ICompilationUnit.UnitType.AS_UNIT
+                && cuType != ICompilationUnit.UnitType.MXML_UNIT)
+        {
+            return;
+        }
+
+        final File outputClassFile = getOutputClassFile(
+                cu.getQualifiedNames().get(0), outputFolder);
+
+        if (config.isVerbose())
+        {
+            System.out.println("Compiling file: " + outputClassFile);
+        }
+
+        ICompilationUnit unit = cu;
+
+        IJSWriter writer;
+        if (cuType == ICompilationUnit.UnitType.AS_UNIT)
+        {
+            writer = (IJSWriter) project.getBackend().createWriter(project,
+                    problems.getProblems(), unit, false);
+        }
+        else
+        {
+            writer = (IJSWriter) project.getBackend().createMXMLWriter(
+                    project, problems.getProblems(), unit, false);
+        }
+
+        BufferedOutputStream out = new BufferedOutputStream(
+                new FileOutputStream(outputClassFile));
+
+        BufferedOutputStream sourceMapOut = null;
+        File outputSourceMapFile = null;
+        if (project.config.getSourceMap())
+        {
+            outputSourceMapFile = getOutputSourceMapFile(
+                    cu.getQualifiedNames().get(0), outputFolder);
+            sourceMapOut = new BufferedOutputStream(
+                    new FileOutputStream(outputSourceMapFile));
+        }
+        
+        writer.writeTo(out, sourceMapOut, outputSourceMapFile);
+        out.flush();
+        out.close();
+        if (sourceMapOut != null)
+        {
+            sourceMapOut.flush();
+            sourceMapOut.close();
+        }
+        writer.close();
+    }
+
     /**
      * Build target artifact.
      * 
@@ -599,7 +688,17 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
 
         ITargetSettings settings = getTargetSettings();
         if (settings != null)
+        {
             project.setTargetSettings(settings);
+        }
+        else
+        {
+            // if getTargetSettings() returned null, then there will definitely
+            // be new config problems that weren't there after applyToProject()
+            // succeeded
+            problems.addAll(projectConfigurator.getConfigurationProblems());
+            return false;
+        }
 
         target = project.getBackend().createTarget(project,
                 getTargetSettings(), null);
@@ -655,7 +754,7 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
             projectConfigurator.applyToProject(project);
             project.config = (JSGoogConfiguration) projectConfigurator.getConfiguration();
 
-            config = projectConfigurator.getConfiguration();
+            config = (JSGoogConfiguration) projectConfigurator.getConfiguration();
             configBuffer = projectConfigurator.getConfigurationBuffer();
 
             problems = new ProblemQuery(projectConfigurator.getCompilerProblemSettings());
@@ -668,6 +767,7 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
                 return false;
 
             validateTargetFile();
+            DefinitionBase.setPerformanceCachingEnabled(!config.getWatch());
             return true;
         }
         catch (ConfigurationException e)
@@ -687,7 +787,7 @@ public class MXMLJSCNode implements JSCompilerEntryPoint, ProblemQueryProvider,
         {
             if (config == null)
             {
-                config = new Configuration();
+                config = new JSConfiguration();
                 configBuffer = new ConfigurationBuffer(Configuration.class,
                         Configuration.getAliases());
             }
