@@ -24,15 +24,7 @@ import static com.google.common.collect.Lists.transform;
 import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 
 import org.apache.commons.io.FileUtils;
 
@@ -66,6 +58,7 @@ import org.apache.royale.compiler.definitions.references.ReferenceFactory;
 import org.apache.royale.compiler.exceptions.LibraryCircularDependencyException;
 import org.apache.royale.compiler.filespecs.IFileSpecification;
 import org.apache.royale.compiler.internal.as.codegen.BindableHelper;
+import org.apache.royale.compiler.internal.codegen.databinding.BindingDatabase;
 import org.apache.royale.compiler.internal.css.CSSManager;
 import org.apache.royale.compiler.internal.css.codegen.CSSCompilationSession;
 import org.apache.royale.compiler.internal.definitions.ClassDefinition;
@@ -76,6 +69,7 @@ import org.apache.royale.compiler.internal.mxml.MXMLDialect;
 import org.apache.royale.compiler.internal.mxml.MXMLManifestManager;
 import org.apache.royale.compiler.internal.mxml.MXMLNamespaceMapping;
 import org.apache.royale.compiler.internal.projects.DependencyGraph.Edge;
+import org.apache.royale.compiler.internal.scopes.ASFileScope;
 import org.apache.royale.compiler.internal.scopes.ASProjectScope;
 import org.apache.royale.compiler.internal.scopes.ASScope;
 import org.apache.royale.compiler.internal.scopes.PackageScope;
@@ -87,6 +81,7 @@ import org.apache.royale.compiler.internal.workspaces.Workspace;
 import org.apache.royale.compiler.mxml.IMXMLLanguageConstants;
 import org.apache.royale.compiler.mxml.IMXMLManifestManager;
 import org.apache.royale.compiler.mxml.IMXMLNamespaceMapping;
+import org.apache.royale.compiler.problems.ICompilerProblem;
 import org.apache.royale.compiler.projects.ICompilerProjectWithNamedColor;
 import org.apache.royale.compiler.projects.IRoyaleProject;
 import org.apache.royale.compiler.scopes.IASScope;
@@ -98,6 +93,7 @@ import org.apache.royale.compiler.targets.ITargetSettings;
 import org.apache.royale.compiler.tree.ASTNodeID;
 import org.apache.royale.compiler.tree.as.IASNode;
 import org.apache.royale.compiler.tree.as.IImportNode;
+import org.apache.royale.compiler.tree.mxml.IMXMLClassDefinitionNode;
 import org.apache.royale.compiler.units.ICompilationUnit;
 import org.apache.royale.compiler.units.requests.IOutgoingDependenciesRequestResult;
 import org.apache.royale.compiler.units.requests.IRequest;
@@ -169,6 +165,11 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
      */
     private String stateClass;
     
+    /**
+     * The resolved stateClass 
+     */
+    public ITypeDefinition stateClassType = null;
+    
     /**
      * The fully-qualified name of the runtime interface 
      * for a component with a <code>currentState</code> property.
@@ -290,6 +291,15 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
      * Currently this is "mx.core.IMXMLObject".
      */
     private String mxmlObjectInterface;
+
+    /**
+     * Whether localId behavior should be 'forced'
+     * for the current compilation target. This has the
+     * effect of treating all mxml id="name" specifications
+     * as localId="name" for the current compilation target.
+     * This is false by default and is managed via Configuration
+     */
+    private Boolean mxmlForceLocalId;
     
     /**
      * The fully-qualified name of the runtime interface
@@ -374,6 +384,7 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
     private String objectProxyClass;
     private String xmlWatcherClass;
     private Name xmlWatcherClassName;
+    private String swfDebugfileAlias;
     
     /**
      * The fully-qualified name of the runtime class
@@ -764,6 +775,16 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         this.mxmlObjectInterface = mxmlObjectInterface;
     }
 
+    public Boolean getMXMLForceLocalId()
+    {
+        return mxmlForceLocalId;
+    }
+
+    public void setMXMLForceLocalId(Boolean mxmlForceLocalId)
+    {
+        this.mxmlForceLocalId = mxmlForceLocalId;
+    }
+
     public String getProxyBaseClass()
     {
         return proxyBaseClass;
@@ -1478,6 +1499,18 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
                 namespaceURIs, true);
     }
 
+
+
+    @Override
+    public void collectProblems(Collection<ICompilerProblem> problems)
+    {
+        super.collectProblems(problems);
+        if (manifestManager != null)
+        {
+            manifestManager.collectProblems(problems);
+        }
+    }
+
     /**
      * Test is a component is 'lookupOnly' or not. If a component is
      * 'lookupOnly' is means this component it not expect to have source in the
@@ -1558,8 +1591,8 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
 //            return;
 //        }
 //        
-//        this.servicesXMLPath = path;
-//        this.servicesContextRoot = contextRoot;
+        this.servicesXMLPath = path;
+        this.servicesContextRoot = contextRoot;
 //        
 //        if (path != null)
 //        {
@@ -1786,6 +1819,8 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         return actionScriptFileEncoding;
     }
 
+    private boolean isTranspiling;
+    
     @Override
     public void setDefineDirectives(Map<String, String> defines)
     {
@@ -1793,6 +1828,10 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         // TODO: This seems strange. Each call to the setter
         // adds new defines. How do you get rid of the old ones?
         addConfigVariables(defines);
+        if (defines.containsKey("COMPILE::SWF"))
+        {
+        	isTranspiling = defines.get("COMPILE::SWF").equalsIgnoreCase("false");
+        }
         clean();
     }
 
@@ -2157,6 +2196,10 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         String thisPackage = null;
         if (scopeDef != null) 
             thisPackage = scopeDef.getPackageName();
+        else if (scope instanceof ASFileScope)
+        {
+        	thisPackage = "";
+        }
         else
         {
             while (!(scope instanceof PackageScope))
@@ -2179,7 +2222,7 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         {
             // now check to see if the class was imported in the window package.
             ASScope pkgScope = (ASScope)scope;
-            while (!(pkgScope instanceof PackageScope))
+            while (!(pkgScope instanceof PackageScope || pkgScope instanceof ASFileScope))
                 pkgScope = pkgScope.getContainingScope();
             String[] imports = pkgScope.getImports();
             String windowName = "window." + name;
@@ -2196,6 +2239,18 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
             // that they meant the one they did import
             if (!usingWindow)
             {
+            	// but if no packages at all first see if it is a local class
+            	if (package1.length() == 0 && package2.length() == 0)
+            	{
+            		if (pkgScope instanceof PackageScope)
+            			pkgScope = pkgScope.getContainingScope();
+            		if (pkgScope.getAllLocalNames().contains(name))
+            		{
+            			IDefinitionSet defSet = pkgScope.getLocalDefinitionSetByName(name);
+            			if (defSet.getSize() == 1)
+            				return defSet.getDefinition(0);
+            		}
+            	}
                 return package1.length() == 0 ? def2 : def1;
             }
             // otherwise fall through to ambiguous because they need to qualify
@@ -2227,9 +2282,10 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
      * it isn't allowed in an ActionScript identifier or MXML id. It also can't
      * conflict with any dynamic properties, because MXML classes are sealed.
      */
-    public String getGeneratedIDBase()
+    public String getGeneratedIDBase(IMXMLClassDefinitionNode definitionNode)
     {
-        return "#";
+        IClassDefinition classDefinition = definitionNode.getDefinition();
+        return "#_"+classDefinition.resolveAncestry(this).length+"_";
     }
     
     @Override
@@ -2320,27 +2376,50 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
 		{
 			if (meta.getTagName().equals(IMetaAttributeConstants.ATTRIBUTE_SWFOVERRIDE))
 			{
-				IMetaTagAttribute attr = meta.getAttribute(IMetaAttributeConstants.NAME_SWFOVERRIDE_ALTPARAMS);
-				if (attr != null)
+				IMetaTagAttribute altattr = meta.getAttribute(IMetaAttributeConstants.NAME_SWFOVERRIDE_ALTPARAMS);
+				if (altattr != null)
 				{
-					// format is expectedQName:allowedQName,expectedQName:allowedQName.
-					// we don't know which parameter it is so we're assuming for now that any mapping
-					// applies to all occurences of that type in the parameter list
-					String paramList = attr.getValue();
-					String[] paramMap;
-					if (paramList.contains(","))
-						paramMap = paramList.split(",");
+					String altparamList = altattr.getValue();
+					String[] altparamMap;
+					if (altparamList.contains(","))
+						altparamMap = altparamList.split(",");
 					else
 					{
-						paramMap = new String[1];
-						paramMap[0] = paramList;
+						altparamMap = new String[1];
+						altparamMap[0] = altparamList;
 					}
-					for (String item : paramMap)
+					IMetaTagAttribute attr = meta.getAttribute(IMetaAttributeConstants.NAME_SWFOVERRIDE_PARAMS);
+					if (attr != null)
 					{
-						String[] parts = item.split(":");
-						if (expectedDefinition.getQualifiedName().equals(parts[0]))
-							if (((ITypeDefinition)actualDefinition).isInstanceOf(parts[1], this))
-								return true;
+						String paramList = attr.getValue();
+						String[] paramMap;
+						if (paramList.contains(","))
+							paramMap = paramList.split(",");
+						else
+						{
+							paramMap = new String[1];
+							paramMap[0] = paramList;
+						}
+						int n = paramMap.length;
+						for (int i = 0; i < n; i++)
+						{
+							String item = paramMap[i];
+							if (((ITypeDefinition)expectedDefinition).isInstanceOf(item, this))
+							{
+								String alts = altparamMap[i];
+								String[] altList;
+								if (alts.contains(":"))
+									altList = alts.split(":");
+								else
+								{
+									altList = new String[1];
+									altList[0] = alts;
+								}
+								for (String alt : altList)
+									if (((ITypeDefinition)actualDefinition).isInstanceOf(alt, this))
+										return true;
+							}
+						}
 					}
 				}
 			}
@@ -2387,6 +2466,126 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
         return list;
     }
 
+    private boolean allowPrivateNameConflicts;
+    
+    /**
+     * List of compiler defines so it can be overridden
+     */
+    @Override
+    public boolean getAllowPrivateNameConflicts()
+    {
+    	return allowPrivateNameConflicts;
+    }
+    public void setAllowPrivateNameConflicts(boolean allow)
+    {
+    	allowPrivateNameConflicts = allow;
+    }
+
+    private boolean allowImportAliases = false;
+    
+    /**
+     * Indicates if import aliases are allowed.
+     */
+    @Override
+    public boolean getAllowImportAliases()
+    {
+    	return allowImportAliases;
+    }
+    public void setAllowImportAliases(boolean allow)
+    {
+    	allowImportAliases = allow;
+    }
+
+    private boolean allowAbstractClasses = false;
+    
+    /**
+     * Indicates if abstract classes are allowed.
+     */
+    @Override
+    public boolean getAllowAbstractClasses()
+    {
+    	return allowAbstractClasses;
+    }
+    public void setAllowAbstractClasses(boolean allow)
+    {
+    	allowAbstractClasses = allow;
+    }
+
+    private boolean allowPrivateConstructors = false;
+    
+    /**
+     * Indicates if private constructors are allowed.
+     */
+    @Override
+    public boolean getAllowPrivateConstructors()
+    {
+    	return allowPrivateConstructors;
+    }
+    public void setAllowPrivateConstructors(boolean allow)
+    {
+    	allowPrivateConstructors = allow;
+    }
+
+    private boolean allowStrictFunctionTypes = false;
+    
+    /**
+     * Indicates if strict function types are allowed and enforced.
+     */
+    @Override
+    public boolean getAllowStrictFunctionTypes()
+    {
+    	return allowStrictFunctionTypes;
+    }
+    public void setAllowStrictFunctionTypes(boolean allow)
+    {
+    	allowStrictFunctionTypes = allow;
+    }
+
+    private boolean strictIdentifierNames = false;
+    
+    /**
+     * Indicates if strict identifier names are enforced.
+     */
+    @Override
+    public boolean getStrictIdentifierNames()
+    {
+    	return strictIdentifierNames;
+    }
+    public void setStrictIdentifierNames(boolean enabled)
+    {
+    	strictIdentifierNames = enabled;
+    }
+
+    private boolean strictFlexCSS = false;
+    
+    /**
+     * Indicates if strict Flex CSS syntax is enforced.
+     */
+    @Override
+    public boolean getStrictFlexCSS()
+    {
+    	return strictFlexCSS;
+    }
+    public void setStrictFlexCSS(boolean enabled)
+    {
+    	strictFlexCSS = enabled;
+    }
+
+    private boolean inferTypes = false;
+
+    /**
+     * Indicates if type inference is enabled.
+     */
+	@Override
+	public boolean getInferTypes()
+    {
+		return inferTypes;
+	}
+    public void setInferTypes(boolean enabled)
+    {
+    	inferTypes = enabled;
+    }
+
 	@Override
 	public boolean isPlatformRule(ICSSRule rule) {
 		return true;
@@ -2429,14 +2628,14 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
 		String qname = cqname + ":" + def.getQualifiedName();
 		if (!apiMap.containsKey(qname))
 		{
-			apiMap.put(qname, new Integer(1));
+			apiMap.put(qname, Integer.valueOf(1));
 			return;
 		}
 		else
 		{
 			Integer counter = apiMap.get(qname);
 			int newCounter = counter.intValue() + 1;
-			apiMap.put(qname, new Integer(newCounter));
+			apiMap.put(qname, Integer.valueOf(newCounter));
 		}
 	}
 	
@@ -2454,15 +2653,59 @@ public class RoyaleProject extends ASProject implements IRoyaleProject, ICompile
 			String qname = def.getQualifiedName();
 			if (!apiMap.containsKey(qname))
 			{
-				apiMap.put(qname, new Integer(1));
+				apiMap.put(qname, Integer.valueOf(1));
 				return;
 			}
 			else
 			{
 				Integer counter = apiMap.get(qname);
 				int newCounter = counter.intValue() + 1;
-				apiMap.put(qname, new Integer(newCounter));
+				apiMap.put(qname, Integer.valueOf(newCounter));
 			}
 		}
 	}
+	
+	@Override
+	public boolean isParameterCountMismatchAllowed(IFunctionDefinition func,
+			int formalCount, int actualCount) {
+		if (!isTranspiling) return false;
+		if (func.getBaseName().equals("sort") &&
+				func.getParent().getQualifiedName().equals("Array") &&
+				formalCount == 1 && actualCount == 2)
+			return true;
+        return false;
+	}
+
+	@Override
+	public File getLinkReport(Configuration config) {
+		return config.getLinkReport();
+	}
+
+    public String getSwfDebugfileAlias()
+    {
+        return swfDebugfileAlias;
+    }
+    
+    public void setSwfDebugfileAlias(String swfDebugfileAlias)
+    {
+        this.swfDebugfileAlias = swfDebugfileAlias;
+    }
+
+    /**
+     * SWF/default Projects have static typing at runtime
+     */
+    public boolean isStaticTypedTarget() {
+	    return true;
+    }
+
+
+    private HashMap<IClassDefinition, BindingDatabase> bindingMap = new HashMap<IClassDefinition, BindingDatabase>();
+    /**
+     * Support for access to BindingData from the class definition as key.
+     * @return
+     */
+    public HashMap<IClassDefinition, BindingDatabase> getBindingMap(){
+        return bindingMap;
+    }
+
 }
